@@ -1,9 +1,12 @@
 from __future__ import annotations
+
 import os
 import datetime
 from typing import ClassVar
+
+from lazy.serialize import Serialize
 from .base import *
-from ..flavours import _async_sync_windows_flavour, _async_sync_posix_flavour
+from ..flavours import _pathz_windows_flavour, _pathz_posix_flavour
 
 
 if TYPE_CHECKING:
@@ -13,9 +16,6 @@ if TYPE_CHECKING:
 
 from .cfs_base import get_accessor, get_cloud_filesystem, AccessorLike, CFSLike
 
-
-#GCP_CFS, GCP_Accessor, _get_gcp_accessor
-
 class PathzCFSPurePath(PurePath):
     _prefix: str = None
     _provider: str = None
@@ -26,8 +26,9 @@ class PathzCFSPurePath(PurePath):
         self._accessor: AccessorLike = get_accessor(self._prefix)
 
     def __new__(cls, *args):
-        if issubclass(PathzCFSPurePath): 
+        if cls is PathzCFSPurePath or issubclass(cls, PathzCFSPurePath):
             cls = cls._win_pathz if os.name == 'nt' else cls._posix_pathz
+            cls = globals()[cls]
         return cls._from_parts(args)
 
     def _new(self, *parts):
@@ -40,7 +41,7 @@ class PurePathzCFSPosixPath(PathzCFSPurePath):
     On a POSIX system, instantiating a PurePath should return this object.
     However, you can also instantiate it directly on any system.
     """
-    _flavour = _async_sync_posix_flavour
+    _flavour = _pathz_posix_flavour
     _pathlike = posixpath
     __slots__ = ()
 
@@ -50,7 +51,7 @@ class PurePathzCFSWindowsPath(PathzCFSPurePath):
     On a Windows system, instantiating a PurePath should return this object.
     However, you can also instantiate it directly on any system.
     """
-    _flavour = _async_sync_windows_flavour
+    _flavour = _pathz_windows_flavour
     _pathlike = ntpath
     __slots__ = ()
 
@@ -59,7 +60,7 @@ class PathzCFSPath(Path, PathzCFSPurePath):
     """
     Our customized class that incorporates both sync and async methods
     """
-    _flavour = _async_sync_windows_flavour if os.name == 'nt' else _async_sync_posix_flavour
+    _flavour = _pathz_windows_flavour if os.name == 'nt' else _pathz_posix_flavour
     _accessor: AccessorLike = None
     _pathlike = posixpath
     _prefix = None
@@ -73,7 +74,7 @@ class PathzCFSPath(Path, PathzCFSPurePath):
         self._fileio = None
 
     def __new__(cls, *parts, **kwargs):
-        if issubclass(PathzCFSPath): 
+        if cls is PathzCFSPath or issubclass(cls, PathzCFSPath): 
             cls = cls._win_pathz if os.name == 'nt' else cls._posix_pathz
             cls = globals()[cls]
         self = cls._from_parts(parts, init=False)
@@ -128,9 +129,18 @@ class PathzCFSPath(Path, PathzCFSPurePath):
         if self._prefix not in self.parts[0]: return self._prefix + '://' + '/'.join(self.parts)
         return self._prefix + '://' + '/'.join(self.parts[1:])
     
+    @property
+    def posix_(self):
+        """Return the string representation of the path with forward (/)
+        slashes."""
+        f = self._flavour
+        return str(self).replace(f.sep, '/')
+
     @property
     def string(self) -> str:
-        return str(self)
+        if self.is_cloud:
+            return self._cloudstr
+        return self.posix_
     
     @property
     def filename_(self) -> str:
@@ -203,7 +213,8 @@ class PathzCFSPath(Path, PathzCFSPurePath):
     
     @property
     def is_cloud(self) -> bool:
-        return bool(self._prefix not in self.parts[0] and self._prefix not in self.parts[1])
+        if not self._prefix: return False
+        return bool(self._prefix in self.parts[0] or self._prefix in self.parts[1])
     
     @property
     def exists_(self) -> bool:
@@ -285,7 +296,7 @@ class PathzCFSPath(Path, PathzCFSPurePath):
         """
         return await self.async_info()
     
-    def open(self, mode: FileMode = 'r', buffering: int = -1, encoding: Optional[str] = DEFAULT_ENCODING, errors: Optional[str] = ON_ERRORS, newline: Optional[str] = NEWLINE, block_size: int = 5242880, compression: str = 'infer', **kwargs: Any) -> IO[Union[str, bytes]]:
+    def open(self, mode: FileMode = 'r', buffering: int = -1, encoding: Optional[str] = DEFAULT_ENCODING, errors: Optional[str] = ON_ERRORS, newline: Optional[str] = NEWLINE, block_size: int = 5242880, compression: str = None, **kwargs: Any) -> IO[Union[str, bytes]]:
         """
         Open the file pointed by this path and return a file object, as
         the built-in open() function does.
@@ -293,10 +304,11 @@ class PathzCFSPath(Path, PathzCFSPurePath):
         return self._accessor.open(self._cloudpath, mode=mode, buffering=buffering, encoding=encoding, errors=errors, newline=newline)
 
     
-    def async_open(self, mode: FileMode = 'r', buffering: int = -1, encoding: Optional[str] = DEFAULT_ENCODING, errors: Optional[str] = ON_ERRORS, newline: Optional[str] = NEWLINE, block_size: int = 5242880, compression: str = 'infer', **kwargs: Any) -> IterableAIOFile:
+    def async_open(self, mode: FileMode = 'r', buffering: int = -1, encoding: Optional[str] = DEFAULT_ENCODING, errors: Optional[str] = ON_ERRORS, newline: Optional[str] = NEWLINE, block_size: int = 5242880, compression: str = None, **kwargs: Any) -> IterableAIOFile:
         """
         Asyncronously Open the file pointed by this path and return a file object, as
         the built-in open() function does.
+        compression = infer doesn't work all that well.
         """
         #self._fileio = self._accessor.open(self._cloudpath, mode=mode, encoding=encoding, errors=errors, block_size=block_size, compression=compression, newline=newline, buffering=buffering, **kwargs)
         #print(type(self._fileio))
@@ -304,35 +316,35 @@ class PathzCFSPath(Path, PathzCFSPurePath):
         return get_cloud_file(self._accessor.open(self._cloudpath, mode=mode, encoding=encoding, errors=errors, block_size=block_size, compression=compression, newline=newline, buffering=buffering, **kwargs))
 
 
-    def reader(self, mode: FileMode = 'r', buffering: int = -1, encoding: Optional[str] = DEFAULT_ENCODING, errors: Optional[str] = ON_ERRORS, newline: Optional[str] = NEWLINE, block_size: int = 5242880, compression: str = 'infer', **kwargs: Any) -> IO[Union[str, bytes]]:
+    def reader(self, mode: FileMode = 'r', buffering: int = -1, encoding: Optional[str] = DEFAULT_ENCODING, errors: Optional[str] = ON_ERRORS, newline: Optional[str] = NEWLINE, block_size: int = 5242880, compression: str = None, **kwargs: Any) -> IO[Union[str, bytes]]:
         """
         Open the file pointed by this path and return a file object, as
         the built-in open() function does.
         """
         return self._accessor.open(self._cloudpath, mode=mode, buffering=buffering, encoding=encoding, errors=errors, block_size=block_size, compression=compression, newline=newline, **kwargs)
     
-    def async_reader(self, mode: FileMode = 'r', buffering: int = -1, encoding: Optional[str] = DEFAULT_ENCODING, errors: Optional[str] = ON_ERRORS, newline: Optional[str] = NEWLINE, block_size: int = 5242880, compression: str = 'infer', **kwargs: Any) -> IterableAIOFile:
+    def async_reader(self, mode: FileMode = 'r', buffering: int = -1, encoding: Optional[str] = DEFAULT_ENCODING, errors: Optional[str] = ON_ERRORS, newline: Optional[str] = NEWLINE, block_size: int = 5242880, compression: str = None, **kwargs: Any) -> IterableAIOFile:
         """
         Asyncronously Open the file pointed by this path and return a file object, as
         the built-in open() function does.
         """
         return get_cloud_file(self._accessor.open(self._cloudpath, mode=mode, buffering=buffering, encoding=encoding, errors=errors, block_size=block_size, compression=compression, newline=newline, **kwargs))
     
-    def appender(self, mode: FileMode = 'a', buffering: int = -1, encoding: Optional[str] = DEFAULT_ENCODING, errors: Optional[str] = ON_ERRORS, newline: Optional[str] = NEWLINE, block_size: int = 5242880, compression: str = 'infer', **kwargs: Any) -> IO[Union[str, bytes]]:
+    def appender(self, mode: FileMode = 'a', buffering: int = -1, encoding: Optional[str] = DEFAULT_ENCODING, errors: Optional[str] = ON_ERRORS, newline: Optional[str] = NEWLINE, block_size: int = 5242880, compression: str = None, **kwargs: Any) -> IO[Union[str, bytes]]:
         """
         Open the file pointed by this path and return a file object, as
         the built-in open() function does.
         """
         return self._accessor.open(self._cloudpath, mode=mode, buffering=buffering, encoding=encoding, errors=errors, block_size=block_size, compression=compression, newline=newline, **kwargs)
     
-    def async_appender(self, mode: FileMode = 'a', buffering: int = -1, encoding: Optional[str] = DEFAULT_ENCODING, errors: Optional[str] = ON_ERRORS, newline: Optional[str] = NEWLINE, block_size: int = 5242880, compression: str = 'infer', **kwargs: Any) -> IterableAIOFile:
+    def async_appender(self, mode: FileMode = 'a', buffering: int = -1, encoding: Optional[str] = DEFAULT_ENCODING, errors: Optional[str] = ON_ERRORS, newline: Optional[str] = NEWLINE, block_size: int = 5242880, compression: str = None, **kwargs: Any) -> IterableAIOFile:
         """
         Asyncronously Open the file pointed by this path and return a file object, as
         the built-in open() function does.
         """
         return get_cloud_file(self._accessor.open(self._cloudpath, mode=mode, buffering=buffering, encoding=encoding, errors=errors, block_size=block_size, compression=compression, newline=newline, **kwargs))
     
-    def writer(self, mode: FileMode = 'w', buffering: int = -1, encoding: Optional[str] = DEFAULT_ENCODING, errors: Optional[str] = ON_ERRORS, newline: Optional[str] = NEWLINE, block_size: int = 5242880, compression: str = 'infer', **kwargs: Any) -> IO[Union[str, bytes]]:
+    def writer(self, mode: FileMode = 'w', buffering: int = -1, encoding: Optional[str] = DEFAULT_ENCODING, errors: Optional[str] = ON_ERRORS, newline: Optional[str] = NEWLINE, block_size: int = 5242880, compression: str = None, **kwargs: Any) -> IO[Union[str, bytes]]:
         """
         Open the file pointed by this path and return a file object, as
         the built-in open() function does.
@@ -340,7 +352,7 @@ class PathzCFSPath(Path, PathzCFSPurePath):
         #self.touch()
         return self._accessor.open(self._cloudpath, mode=mode, buffering=buffering, encoding=encoding, errors=errors, block_size=block_size, compression=compression, newline=newline, **kwargs)
     
-    def async_writer(self, mode: FileMode = 'w', buffering: int = -1, encoding: Optional[str] = DEFAULT_ENCODING, errors: Optional[str] = ON_ERRORS, newline: Optional[str] = NEWLINE, block_size: int = 5242880, compression: str = 'infer', **kwargs: Any) -> IterableAIOFile:
+    def async_writer(self, mode: FileMode = 'w', buffering: int = -1, encoding: Optional[str] = DEFAULT_ENCODING, errors: Optional[str] = ON_ERRORS, newline: Optional[str] = NEWLINE, block_size: int = 5242880, compression: str = None, **kwargs: Any) -> IterableAIOFile:
         """
         Asyncronously Open the file pointed by this path and return a file object, as
         the built-in open() function does.
@@ -1344,8 +1356,8 @@ __all__ = (
     'PurePathzCFSWindowsPath',
     'PathzCFSPath',
     'Path',
-    '_async_sync_windows_flavour',
-    '_async_sync_posix_flavour',
+    '_pathz_windows_flavour',
+    '_pathz_posix_flavour',
     'PathzCFSPosixPath',
     'PathzCFSWindowsPath',
     'register_pathlike'
