@@ -1,5 +1,7 @@
 from __future__ import annotations
+
 import os
+
 from .base import *
 from .cfs_base import Minio_CFS
 from .cfs_pathz_base import *
@@ -17,7 +19,7 @@ class PurePathzMinioPosixPath(PurePathzCFSPosixPath):
     On a POSIX system, instantiating a PurePath should return this object.
     However, you can also instantiate it directly on any system.
     """
-    _flavour = _async_sync_posix_flavour
+    _flavour = _pathz_posix_flavour
     _pathlike = posixpath
     __slots__ = ()
 
@@ -27,7 +29,7 @@ class PurePathzMinioWindowsPath(PurePathzCFSWindowsPath):
     On a Windows system, instantiating a PurePath should return this object.
     However, you can also instantiate it directly on any system.
     """
-    _flavour = _async_sync_windows_flavour
+    _flavour = _pathz_windows_flavour
     _pathlike = ntpath
     __slots__ = ()
 
@@ -35,7 +37,7 @@ class PathzMinioPath(PathzCFSPath):
     """
     Our customized class that incorporates both sync and async methods
     """
-    _flavour = _async_sync_windows_flavour if os.name == 'nt' else _async_sync_posix_flavour
+    _flavour = _pathz_windows_flavour if os.name == 'nt' else _pathz_posix_flavour
     _accessor: AccessorLike = None
     _pathlike = posixpath
     _prefix = 's3'
